@@ -149,8 +149,10 @@ void cmdline_completion(vector_obj* all_candidate, char* editing, BOOL add_space
         int l = strlen(editing);
 
         const int len = strlen(string_c_str(vector_item(gCCandidate, 0)));
-        char c = 0;
-        BOOL kanji = FALSE;
+        unsigned char c = 0;
+        char utf8[MB_CUR_MAX];
+        int utf8_len = 0;
+        int utf8_size = -1;
         while(l < len) {
             BOOL same = TRUE;
 
@@ -166,35 +168,45 @@ void cmdline_completion(vector_obj* all_candidate, char* editing, BOOL add_space
             if(same) {
                 char tmp[256];
                 c = (string_c_str(vector_item(gCCandidate, 0)))[l];
-                
-                BOOL kanji_before = kanji;
-                if(gKanjiCode != kUtf8) {
-                    if(kanji) {
-                        kanji = FALSE;
-                    }
-                    else {
-                        if(is_kanji(kUtf8, c)) {
-                            kanji = TRUE;
-                        }
-                    }
+
+                if(utf8_size == -1 && c >= 128) {
+                    utf8_size = ((c & 0x80) >> 7) + ((c & 0x40) >> 6) + ((c & 0x20) >> 5) + ((c & 0x10) >> 4);
                 }
 
-                if(!gCompletionSqort && !gCompletionDqort && (gKanjiCode==kUtf8 || !kanji_before) && (c==' ' || c=='!' || c=='*' || c=='>' || c=='&' || c=='~' || c=='#' || c =='$' || c=='(' || c==')' || c=='\\' || c=='|' || c=='[' || c==']' || c=='{' || c=='}' || c==';' || c=='\'' || c=='"' || c=='<' || c=='>' || c=='?'))
-                {
-                    sprintf(tmp, "\\%c", c);
-                    string_insert(str, gCmdLineCursor, tmp);
-                    gCmdLineCursor+=2;
+                if(utf8_size != -1) {
+                    if(utf8_len < utf8_size) {
+                        utf8[utf8_len++] = c;
+                    }
 
-                    gEditingLen++;
-                    sprintf(gEditing + strlen(gEditing), "%c", c);
+                    if(utf8_len == utf8_size) {
+                        utf8[utf8_len] = 0;
+                        sprintf(tmp, "%s", utf8);
+                        string_insert(str, gCmdLineCursor, tmp);
+                        gCmdLineCursor+=utf8_len;
+                        gEditingLen+=utf8_len;
+                        strcat(gEditing, tmp);
+                        utf8_size = -1;
+                        utf8_len = 0;
+                    }
                 }
                 else {
-                    sprintf(tmp, "%c", c);
-                    string_insert(str, gCmdLineCursor, tmp);
-                    gCmdLineCursor++;
+                    if(!gCompletionSqort && !gCompletionDqort && (c==' ' || c=='!' || c=='*' || c=='>' || c=='&' || c=='~' || c=='#' || c =='$' || c=='(' || c==')' || c=='\\' || c=='|' || c=='[' || c==']' || c=='{' || c=='}' || c==';' || c=='\'' || c=='"' || c=='<' || c=='>' || c=='?'))
+                    {
+                        sprintf(tmp, "\\%c", c);
+                        string_insert(str, gCmdLineCursor, tmp);
+                        gCmdLineCursor+=2;
 
-                    gEditingLen++;
-                    strcat(gEditing, tmp);
+                        gEditingLen++;
+                        sprintf(gEditing + strlen(gEditing), "%c", c);
+                    }
+                    else {
+                        sprintf(tmp, "%c", c);
+                        string_insert(str, gCmdLineCursor, tmp);
+                        gCmdLineCursor++;
+
+                        gEditingLen++;
+                        strcat(gEditing, tmp);
+                    }
                 }
             }
             else {
@@ -204,13 +216,6 @@ void cmdline_completion(vector_obj* all_candidate, char* editing, BOOL add_space
             l++;
         }
 
-        if(kanji) {
-            gCmdLineCursor--;
-            string_erase(str, gCmdLineCursor, 1);
-            gEditingLen--;
-            gEditing[strlen(gEditing)-1] = 0;
-        }
-
         if(l == len && vector_size(gCCandidate) == 1) {
             if(c != '/') {
                 if(gCompletionSqort && !gCompletionLastSqort) {
@@ -391,10 +396,8 @@ void cmdline_completion_file(char* editing_dir, char* editing_file, char* editin
             char *p = editing_dir2;
             p++;
             
-            
             char *p2 = env_name;
             
-            
             while(*p != '/') {
                 *p2++ = *p++;                
             }
@@ -521,7 +524,96 @@ void cmdline_completion_program(char* editing_dir, char* editing_file, char* edi
         cmdline_completion_file(editing_dir, editing_file, editing, editing_position);
     }
     else {
-        cmdline_completion(gPrograms, editing, TRUE);
+        cmdline_completion(gSaphireProgCompletions, editing, TRUE);
+    }
+}
+
+///////////////////////////////////////////////////
+// 変数の補完
+///////////////////////////////////////////////////
+void cmdline_completion_var(char* editing)
+{
+    saphire_make_var_completion_list();
+    
+    if(*editing == '$' && *(editing+1) !='$') {
+        vector_obj* new_candidate = VECTOR_NEW(100);
+        
+        int i;
+        for(i=0; i<vector_size(gSaphireVarCompletions); i++) {
+            string_obj* var = vector_item(gSaphireVarCompletions, i);
+            
+            string_obj* new_var = STRING_NEW(string_c_str(var));
+            string_insert(new_var, 0, "$");
+
+            vector_add(new_candidate, new_var);
+        }
+
+        cmdline_completion(new_candidate, editing, TRUE);
+
+        for(i=0; i<vector_size(new_candidate); i++) {
+            string_delete(vector_item(new_candidate, i));
+        }
+        vector_delete(new_candidate);
+    }
+    else if(*editing == '$' && *(editing+1) =='$') {
+        vector_obj* new_candidate = VECTOR_NEW(100);
+        
+        int i;
+        for(i=0; i<vector_size(gSaphireVarCompletions); i++) {
+            string_obj* var = vector_item(gSaphireVarCompletions, i);
+            
+            string_obj* new_var = STRING_NEW(string_c_str(var));
+            string_insert(new_var, 0, "$$");
+
+            vector_add(new_candidate, new_var);
+        }
+
+        cmdline_completion(new_candidate, editing, TRUE);
+
+        for(i=0; i<vector_size(new_candidate); i++) {
+            string_delete(vector_item(new_candidate, i));
+        }
+        vector_delete(new_candidate);
+    }
+    else if(*editing == '@' && *(editing+1) !='@') {
+        vector_obj* new_candidate = VECTOR_NEW(100);
+        
+        int i;
+        for(i=0; i<vector_size(gSaphireVarCompletions); i++) {
+            string_obj* var = vector_item(gSaphireVarCompletions, i);
+            
+            string_obj* new_var = STRING_NEW(string_c_str(var));
+            string_insert(new_var, 0, "@");
+
+            vector_add(new_candidate, new_var);
+        }
+
+        cmdline_completion(new_candidate, editing, TRUE);
+
+        for(i=0; i<vector_size(new_candidate); i++) {
+            string_delete(vector_item(new_candidate, i));
+        }
+        vector_delete(new_candidate);
+    }
+    else if(*editing == '@' && *(editing+1) =='@') {
+        vector_obj* new_candidate = VECTOR_NEW(100);
+        
+        int i;
+        for(i=0; i<vector_size(gSaphireVarCompletions); i++) {
+            string_obj* var = vector_item(gSaphireVarCompletions, i);
+            
+            string_obj* new_var = STRING_NEW(string_c_str(var));
+            string_insert(new_var, 0, "@@");
+
+            vector_add(new_candidate, new_var);
+        }
+
+        cmdline_completion(new_candidate, editing, TRUE);
+
+        for(i=0; i<vector_size(new_candidate); i++) {
+            string_delete(vector_item(new_candidate, i));
+        }
+        vector_delete(new_candidate);
     }
 }
 
@@ -709,7 +801,10 @@ void cmdline_completion_main()
         gCompletionLastSqort = last_squote;
         gCompletionLastDqort = last_dquote;
 
-        if(editing_position == 0) {
+        if(*editing == '$' || *editing == '@') {
+            cmdline_completion_var(editing);
+        }
+        else if(editing_position == 0) {
             cmdline_completion_program(editing_dir, editing_file, editing
                                 , editing_position);
         }
@@ -768,9 +863,10 @@ void cmdline_completion_view()
             else {
                 editing_len = strlen(gEditing);
             }
-            int editing_len2 = strlen(gEditing);
+            //int editing_len2 = strlen(gEditing);
 
             if(editing_len < maxx/gCmdLineCompletionMaxRow-1) {
+/*
                 char tmp[4096];
                 str_cut2(kUtf8, item, editing_len, tmp, 4096);
 
@@ -792,9 +888,18 @@ void cmdline_completion_view()
 
                     mmvprintw(line, row * (maxx/gCmdLineCompletionMaxRow)
                                         , "%s", tmp);
-                    mattroff();
+                    //mattroff();
                     mprintw("%s", tmp2);
                 }
+*/
+                if(gCompletionCursor == i) {
+                    mattron(kCAReverse);
+                    mmvprintw(line, row * (maxx/gCmdLineCompletionMaxRow) , "%s", item);
+                    mattroff();
+                }
+                else {
+                    mmvprintw(line, row * (maxx/gCmdLineCompletionMaxRow) , "%s", item);
+                }
 
                 row++;
             }
@@ -808,13 +913,7 @@ void cmdline_completion_view()
                     mattroff();
                 }
                 else {
-                    char* env = getenv("CMDLINE_COLOR");
-                    if(env && strcmp(env, "1") == 0) 
-                        mattron(kCACyan); 
-                    else 
-                        mattron(kCABold);
                     mmvprintw(line, row * (maxx/gCmdLineCompletionMaxRow), "%s", tmp);
-                    mattroff();
                 }
 
                 row++;
