@@ -101,7 +101,6 @@ BOOL commands_cursor_move(int* rcode, vector_obj* argv, vector_obj* blocks, vect
             }
             string_delete(vector_item(argv, l));
             vector_erase(argv, l);
-            l--;
             string_delete(vector_item(argv, l));
             vector_erase(argv, l);
             l--;
@@ -202,7 +201,6 @@ BOOL commands_mcd(int* rcode, vector_obj* argv, vector_obj* blocks, vector_obj*
             }
             string_delete(vector_item(argv, l));
             vector_erase(argv, l);
-            l--;
             string_delete(vector_item(argv, l));
             vector_erase(argv, l);
             l--;
@@ -283,7 +281,6 @@ BOOL commands_file_name(int* rcode, vector_obj* argv, vector_obj* blocks, vector
             }
             string_delete(vector_item(argv, l));
             vector_erase(argv, l);
-            l--;
             string_delete(vector_item(argv, l));
             vector_erase(argv, l);
             l--;
@@ -336,7 +333,6 @@ BOOL commands_path(int* rcode, vector_obj* argv, vector_obj* blocks, vector_obj*
             }
             string_delete(vector_item(argv, l));
             vector_erase(argv, l);
-            l--;
             string_delete(vector_item(argv, l));
             vector_erase(argv, l);
             l--;
@@ -386,7 +382,6 @@ BOOL commands_file_ext(int* rcode, vector_obj* argv, vector_obj* blocks, vector_
             }
             string_delete(vector_item(argv, l));
             vector_erase(argv, l);
-            l--;
             string_delete(vector_item(argv, l));
             vector_erase(argv, l);
             l--;
@@ -440,7 +435,6 @@ BOOL commands_file_index(int* rcode, vector_obj* argv, vector_obj* blocks, vecto
             }
             string_delete(vector_item(argv, l));
             vector_erase(argv, l);
-            l--;
             string_delete(vector_item(argv, l));
             vector_erase(argv, l);
             l--;
@@ -492,7 +486,6 @@ BOOL commands_file_user(int* rcode, vector_obj* argv, vector_obj* blocks, vector
             }
             string_delete(vector_item(argv, l));
             vector_erase(argv, l);
-            l--;
             string_delete(vector_item(argv, l));
             vector_erase(argv, l);
             l--;
@@ -546,7 +539,6 @@ BOOL commands_file_group(int* rcode, vector_obj* argv, vector_obj* blocks, vecto
             }
             string_delete(vector_item(argv, l));
             vector_erase(argv, l);
-            l--;
             string_delete(vector_item(argv, l));
             vector_erase(argv, l);
             l--;
@@ -600,7 +592,6 @@ BOOL commands_file_perm(int* rcode, vector_obj* argv, vector_obj* blocks, vector
             }
             string_delete(vector_item(argv, l));
             vector_erase(argv, l);
-            l--;
             string_delete(vector_item(argv, l));
             vector_erase(argv, l);
             l--;
@@ -658,7 +649,6 @@ BOOL commands_file_num(int* rcode, vector_obj* argv, vector_obj* blocks, vector_
             }
             string_delete(vector_item(argv, l));
             vector_erase(argv, l);
-            l--;
             string_delete(vector_item(argv, l));
             vector_erase(argv, l);
             l--;
@@ -725,7 +715,6 @@ BOOL commands_cursor_num(int* rcode, vector_obj* argv, vector_obj* blocks, vecto
             }
             string_delete(vector_item(argv, l));
             vector_erase(argv, l);
-            l--;
             string_delete(vector_item(argv, l));
             vector_erase(argv, l);
             l--;
@@ -1030,7 +1019,6 @@ BOOL commands_reread(int* rcode, vector_obj* argv, vector_obj* blocks, vector_ob
             }
             string_delete(vector_item(argv, l));
             vector_erase(argv, l);
-            l--;
             string_delete(vector_item(argv, l));
             vector_erase(argv, l);
             l--;
@@ -1080,7 +1068,6 @@ BOOL commands_allfiles(int* rcode, vector_obj* argv, vector_obj* blocks, vector_
             }
             string_delete(vector_item(argv, l));
             vector_erase(argv, l);
-            l--;
             string_delete(vector_item(argv, l));
             vector_erase(argv, l);
             l--;
@@ -1289,7 +1276,6 @@ BOOL commands_markfiles(int* rcode, vector_obj* argv, vector_obj* blocks, vector
             }
             string_delete(vector_item(argv, l));
             vector_erase(argv, l);
-            l--;
             string_delete(vector_item(argv, l));
             vector_erase(argv, l);
             l--;
@@ -1583,7 +1569,6 @@ BOOL commands_mark(int* rcode, vector_obj* argv, vector_obj* blocks, vector_obj*
             }
             string_delete(vector_item(argv, l));
             vector_erase(argv, l);
-            l--;
             string_delete(vector_item(argv, l));
             vector_erase(argv, l);
             l--;
@@ -1733,12 +1718,49 @@ BOOL commands_mark(int* rcode, vector_obj* argv, vector_obj* blocks, vector_obj*
 
 BOOL commands_new_dir(int* rcode, vector_obj* argv, vector_obj* blocks, vector_obj* parent_blocks, sWFd* nextout, sRFd* nextin, int nexterr, char* title, BOOL input)
 {
+    int l;
+    string_obj* mask = NULL;
+    BOOL dotdir_mask = FALSE;
+    for(l=0; l<vector_size(argv); l++) {
+        char* arg = string_c_str(vector_item(argv, l));
+
+        if(strcmp(arg, "-m") == 0 && l+1 < vector_size(argv)) {
+            char* darg = string_c_str(vector_item(argv, l+1));
+            mask = STRING_NEW(darg);
+
+            string_delete(vector_item(argv, l));
+            vector_erase(argv, l);
+            string_delete(vector_item(argv, l));
+            vector_erase(argv, l);
+            l--;
+            continue;
+        }
+        else if(strcmp(arg, "-dotdir") == 0 && l+1 < vector_size(argv)) {
+            char* darg = string_c_str(vector_item(argv, l+1));
+            dotdir_mask = atoi(darg);
+
+            string_delete(vector_item(argv, l));
+            vector_erase(argv, l);
+            string_delete(vector_item(argv, l));
+            vector_erase(argv, l);
+            l--;
+            continue;
+        }
+    }
+
     if(vector_size(argv) == 2) {
         char* dir = string_c_str(vector_item(argv, 1));
-        filer_new_dir(dir);
+        int dir2 = filer_new_dir(dir);
+        filer_set_dotdir_mask(dir2, dotdir_mask);
+        if(mask) {
+            filer_set_mask(dir2, string_c_str(mask));
+        }
+        filer_cd(dir2, dir);
         *rcode = 0;
     }
 
+    if(mask) string_delete(mask);
+
     return TRUE;
 }
 
@@ -1765,7 +1787,6 @@ BOOL commands_del_dir(int* rcode, vector_obj* argv, vector_obj* blocks, vector_o
             }
             string_delete(vector_item(argv, l));
             vector_erase(argv, l);
-            l--;
             string_delete(vector_item(argv, l));
             vector_erase(argv, l);
             l--;
@@ -1812,7 +1833,6 @@ BOOL commands_marking(int* rcode, vector_obj* argv, vector_obj* blocks, vector_o
             }
             string_delete(vector_item(argv, l));
             vector_erase(argv, l);
-            l--;
             string_delete(vector_item(argv, l));
             vector_erase(argv, l);
             l--;
@@ -1850,6 +1870,7 @@ BOOL commands_mask(int* rcode, vector_obj* argv, vector_obj* blocks, vector_obj*
     int l;
     int dir = adir();
     BOOL dotdir = FALSE;
+    BOOL reread = FALSE;
     for(l=0; l<vector_size(argv); l++) {
         char* arg = string_c_str(vector_item(argv, l));
 
@@ -1869,7 +1890,6 @@ BOOL commands_mask(int* rcode, vector_obj* argv, vector_obj* blocks, vector_obj*
             }
             string_delete(vector_item(argv, l));
             vector_erase(argv, l);
-            l--;
             string_delete(vector_item(argv, l));
             vector_erase(argv, l);
             l--;
@@ -1882,6 +1902,13 @@ BOOL commands_mask(int* rcode, vector_obj* argv, vector_obj* blocks, vector_obj*
             l--;
             continue;
         }
+        else if(strcmp(arg, "-r") == 0) {
+            reread = TRUE;
+            string_delete(vector_item(argv, l));
+            vector_erase(argv, l);
+            l--;
+            continue;
+        }
     }
 
     if(dotdir) {
@@ -1950,16 +1977,20 @@ BOOL commands_mask(int* rcode, vector_obj* argv, vector_obj* blocks, vector_obj*
                     filer_set_mask(i, string_c_str(vector_item(argv, 1)));
                 }
 
-                filer_reread(0);
-                filer_reread(1);
+                if(reread) {
+                    filer_reread(0);
+                    filer_reread(1);
+                }
             }
             else {
                 filer_set_mask(dir, string_c_str(vector_item(argv, 1)));
 
-                if(dir == 0)
-                    filer_reread(0);
-                else if(dir == 1) 
-                    filer_reread(1);
+                if(reread) {
+                    if(dir == 0)
+                        filer_reread(0);
+                    else if(dir == 1) 
+                        filer_reread(1);
+                }
             }
 
             *rcode = 0;
