@@ -192,9 +192,30 @@ void view(BOOL cmdline_draw)
 ///////////////////////////////////////////////////
 static void read_rc_file_compiled()
 {
-    char rc_fname[256];
+    char rc_fname[PATH_MAX];
 
-    sprintf(rc_fname, "%s/mfiler3.sao", gHomeDir);
+    snprintf(rc_fname, PATH_MAX, "%s/mfiler3.sao", SYSCONFDIR);
+    if(access(rc_fname, R_OK) == 0) {
+        sRFd* pipein = sRFd_new(STDIN_FILENO);
+        sWFd* pipeout = sWFd_new(STDOUT_FILENO);
+        int rcode = saphire_load_obj(rc_fname
+            , pipeout, pipein, STDERR_FILENO, NULL);
+        saphire_sweep();
+        sRFd_delete(pipein);
+        sWFd_flash(pipeout);
+        sWFd_delete(pipeout);
+
+        if(rcode < 0) {
+            fprintf(stderr, "%s\n", string_c_str(gErrMsg));
+            exit(1);
+        }
+    }
+    else {
+        fprintf(stderr, "can't find %s/mfiler3.sao file\n", SYSCONFDIR);
+        exit(1);
+    }
+
+    snprintf(rc_fname, PATH_MAX, "%s/mfiler3.sao", gHomeDir);
 
     if(access(rc_fname, R_OK) == 0) {
         sRFd* pipein = sRFd_new(STDIN_FILENO);
@@ -210,29 +231,6 @@ static void read_rc_file_compiled()
             exit(1);
         }
     }
-    else {
-        sprintf(rc_fname,"%s/mfiler3.sao", SYSCONFDIR);
-
-        if(access(rc_fname, R_OK) == 0) {
-            sRFd* pipein = sRFd_new(STDIN_FILENO);
-            sWFd* pipeout = sWFd_new(STDOUT_FILENO);
-            int rcode = saphire_load_obj(rc_fname
-                , pipeout, pipein, STDERR_FILENO, NULL);
-            saphire_sweep();
-            sRFd_delete(pipein);
-            sWFd_flash(pipeout);
-            sWFd_delete(pipeout);
-
-            if(rcode < 0) {
-                fprintf(stderr, "%s\n", string_c_str(gErrMsg));
-                exit(1);
-            }
-        }
-        else {
-            fprintf(stderr, "can't find %s/mfiler3.sao file\n", SYSCONFDIR);
-            exit(1);
-        }
-    }
 }
 
 static void read_rc_file()
@@ -889,7 +887,7 @@ int main(int argc, char* argv[])
     CHECKML_BEGIN();
 
     /// ファイラでもスクリプトでも共通の環境変数を初期化 ///
-    setenv("VERSION", "4.0.9", 1);
+    setenv("VERSION", "4.1.1", 1);
 
     /// mfiler3のホームディレクトリを保存しておく ///
     char* home = getenv("HOME");
@@ -900,10 +898,9 @@ int main(int argc, char* argv[])
     sprintf(gHomeDir, "%s/.mfiler3", home);
 
     if(access(gHomeDir, F_OK) != 0) {
-        if(mkdir(gHomeDir, 0700) < 0) {
-            fprintf(stderr, "can't mkdir %s", gHomeDir);
-            exit(1);
-        }
+	char buf[PATH_MAX];
+        snprintf(buf, PATH_MAX, "mkdir -p %s", gHomeDir);
+        system(buf);
     }
 
     if(chmod(gHomeDir, S_IRUSR|S_IWUSR|S_IXUSR) < 0) {
@@ -1125,13 +1122,12 @@ int main(int argc, char* argv[])
     /// モジュールの初期化 ///
     cmdline_init();
     menu_init();
-    menu_init();
     filer_init();
     mcurses_init(kTKUtf8);
     kanji_init();
     saphire_set_signal();
     saphire_init(kATConsoleApp, TRUE, kRSObject, FALSE);
-    saphire_rehash();
+    //saphire_rehash();
     cmdline_completion_init();
     commands_init();
 
@@ -1183,27 +1179,46 @@ int main(int argc, char* argv[])
     //read_rc_file();
     read_rc_file_compiled();
 
+    /// ディレクトリを作成 ///
+    if(vector_size(gDirs) == 0) {
+        if(strcmp(file_name, "") == 0) {
+            char cwd[PATH_MAX];
+            mygetcwd(cwd);
+
+            filer_new_dir(cwd);
+            filer_cd(0, string_c_str(filer_dir(0)->mPath));
+            filer_new_dir(cwd);
+            filer_cd(1, string_c_str(filer_dir(1)->mPath));
+        }
+        else {
+            filer_new_dir(file_name);
+            filer_cd(0, string_c_str(filer_dir(0)->mPath));
+            filer_new_dir(file_name);
+            filer_cd(1, string_c_str(filer_dir(1)->mPath));
+        }
+    }
+    else if(vector_size(gDirs) == 1) {
+        if(strcmp(file_name, "") == 0) {
+            char cwd[PATH_MAX];
+            mygetcwd(cwd);
+
+            filer_new_dir(cwd);
+            filer_cd(0, string_c_str(filer_dir(0)->mPath));
+        }
+        else {
+            filer_new_dir(file_name);
+            filer_cd(0, string_c_str(filer_dir(0)->mPath));
+        }
+    }
+
+    filer_activate(0);
+
     /// リハッシュ ///
     //saphire_job_done = ;
     saphire_rehash();
 
     /// モジュールの初期化(ランタイムスクリプト以降の方が良い奴 ///
     isearch_init();
-
-    /// ディレクトリを作成 ///
-    if(strcmp(file_name, "") == 0) {
-        char cwd[PATH_MAX];
-        mygetcwd(cwd);
-
-        filer_new_dir(cwd);
-        filer_new_dir(cwd);
-    }
-    else {
-        filer_new_dir(file_name);
-        filer_new_dir(file_name);
-    }
-
-    filer_activate(0);
     
     /// カーシスの起動 ///
     minitscr();
@@ -1239,6 +1254,7 @@ int main(int argc, char* argv[])
             }
         }
 
+
         /// キー処理 ///
         if(FD_ISSET(0, &read_ok) || mkbuf_exist()) {    // キー入力があったか、キーボードのバッファがあるなら
             /// コマンドライン入力中 ///
